package core.services;

import core.api.models.timemanager.taskSchedule.BasicTaskScheduleFieldInfo;
import core.entities.User;
import core.entities.timemanager.BasicTaskSchedule;
import core.entities.timemanager.Task;
import core.repositories.UserRepository;
import core.repositories.timemanager.BasicTaskScheduleRepository;
import core.repositories.timemanager.TaskRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

@Service
public class TaskScheduleService {

    private final BasicTaskScheduleRepository basicTaskScheduleRepository;
    private final TaskRepository taskRepository;
    private final UserRepository userRepository;

    public TaskScheduleService(@Autowired BasicTaskScheduleRepository basicTaskScheduleRepository,
                               @Autowired TaskRepository taskRepository,
                               UserRepository userRepository) {
        this.basicTaskScheduleRepository = basicTaskScheduleRepository;
        this.taskRepository = taskRepository;
        this.userRepository = userRepository;
    }

    public ServiceResult<BasicTaskSchedule> createBasicTaskSchedule(Task task, BasicTaskScheduleFieldInfo basicTaskScheduleFieldInfo) {
        BasicTaskSchedule basicTaskSchedule = new BasicTaskSchedule(task, basicTaskScheduleFieldInfo.getScheduleDate());
        task.getBasicTaskSchedules().add(basicTaskSchedule);
        basicTaskScheduleRepository.save(basicTaskSchedule);
        taskRepository.save(task);
        return new ServiceResult<>(basicTaskSchedule);
    }

    public PermissionResult<BasicTaskSchedule> getSchedulePermissions(long scheduleID, String name) {
        Optional<BasicTaskSchedule> basicSchedule = basicTaskScheduleRepository.findById(scheduleID);
        return basicSchedule.map(value -> new PermissionResult<>(value, value.getTask().getTaskgroup().getUser().getUsername().equals(name))).orElseGet(() ->
                new PermissionResult<>(ServiceExitCode.MISSING_ENTITY));
    }

    public void reschedule(BasicTaskSchedule basicTaskSchedule, BasicTaskScheduleFieldInfo basicTaskScheduleFieldInfo) {
        basicTaskSchedule.setScheduleDate(basicTaskScheduleFieldInfo.getScheduleDate());
        basicTaskScheduleRepository.save(basicTaskSchedule);
    }

    public void deleteBasicSchedule(BasicTaskSchedule basicTaskSchedule) {
        basicTaskScheduleRepository.deleteBasicTaskScheduleByID(basicTaskSchedule.getScheduleID());
    }

    public ServiceResult<List<BasicTaskSchedule>> loadTodaysSchedule(String username, boolean onlyActivateable) {
        Optional<User> user = userRepository.findByUsername(username);
        if(user.isEmpty()) {
            return new ServiceResult<>(ServiceExitCode.MISSING_ENTITY);
        }

        List<BasicTaskSchedule> basicTaskSchedules = basicTaskScheduleRepository.findAllByUserAndDate(user.get(), LocalDate.now());
        if(onlyActivateable) {
            for(int i=0; i<basicTaskSchedules.size(); i++)  {
                if(!basicTaskSchedules.get(i).isActivateAble()) {
                    basicTaskSchedules.remove(i);
                }
            }
        }
        return new ServiceResult<>(basicTaskSchedules);
    }

    public ServiceResult<List<BasicTaskSchedule>> loadSchedules(String username) {
        Optional<User> user = userRepository.findByUsername(username);
        return user.map(value -> new ServiceResult<>(basicTaskScheduleRepository.findAllByUser(value))).orElseGet(() ->
                new ServiceResult<>(ServiceExitCode.MISSING_ENTITY));
    }

    public ServiceResult<BasicTaskSchedule> scheduleTaskNow(Task task) {
        //Check if task has already an active schedule
        if(task.hasActiveSchedule()) {
            return new ServiceResult<>(ServiceExitCode.ENTITY_ALREADY_EXIST);
        } else {
            BasicTaskSchedule basicTaskSchedule = new BasicTaskSchedule(task, LocalDate.now());
            task.getBasicTaskSchedules().add(basicTaskSchedule);
            basicTaskScheduleRepository.save(basicTaskSchedule);
            taskRepository.save(task);
            return new ServiceResult<>(basicTaskSchedule);
        }
    }

    public void deleteScheduleByTask(Task task) {
        basicTaskScheduleRepository.deleteAllByTask(task);
    }

    public ServiceResult<BasicTaskSchedule> getActiveSchedule(String username) {
        Optional<BasicTaskSchedule> activeTaskScheduleResult = basicTaskScheduleRepository.findActiveTaskSchedule(username);
        return activeTaskScheduleResult.map(ServiceResult::new).orElseGet(() -> new ServiceResult<>(ServiceExitCode.MISSING_ENTITY));
    }

    public ServiceResult<BasicTaskSchedule> activateSchedule(BasicTaskSchedule taskSchedule) {
        //Check if taskSchedule can be started
        if(taskSchedule.getStartTime() != null) {
            return new ServiceResult<>(ServiceExitCode.INVALID_OPERATION);
        }

        if(getActiveSchedule(taskSchedule.getTask().getTaskgroup().getUser().getUsername()).getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            taskSchedule.setStartTime(LocalDateTime.now());
            basicTaskScheduleRepository.save(taskSchedule);
            return new ServiceResult<>(taskSchedule);
        } else {
            return new ServiceResult<>(ServiceExitCode.ENTITY_ALREADY_EXIST);
        }
    }

    public ServiceResult<Integer> stopSchedule(BasicTaskSchedule taskSchedule) {
        if(taskSchedule.getStartTime() == null || taskSchedule.getFinishedTime() != null) {
            return new ServiceResult<>(ServiceExitCode.INVALID_OPERATION);
        }

        taskSchedule.setFinishedTime(LocalDateTime.now());
        long activeTime = Duration.between(taskSchedule.getStartTime(), taskSchedule.getFinishedTime()).toMinutes();
        long workTime = (taskSchedule.getTask().getWorkTime() + activeTime);
        int workTime_i = (int) (workTime);
        taskSchedule.getTask().setWorkTime(workTime_i);
        basicTaskScheduleRepository.save(taskSchedule);
        taskRepository.save(taskSchedule.getTask());
        return new ServiceResult<>(workTime_i);
    }
}
